MutexGuard

Struct MutexGuard 

pub struct MutexGuard<'a, P: Priority, T: ?Sized, Q: MutexQueue<P>> { /* private fields */ }
A guard holding access for a Mutex. When dropped, the lock is released.

If the evict flag is enabled, higher priority requesters will mark held locks for eviction, which can be subscribed to via Self::evicted (associated function).

Implementations

Source

impl<'a, P: Priority, T: ?Sized, Q: MutexQueue<P>> MutexGuard<'a, P, T, Q>

pub fn evicted(this: &mut Self) -> impl Future<Output = ()>

Returns a future which resolves when/if another, higher priority holder attempts to acquire the lock.

Note: this is an associated method to avoid colision with methods of T. Invoke via MutexGuard::evicted(&mut self).

Cancel Safety: This function is cancel safe.