Module priority

Module priority 

Contains the Priority trait responsible for establishing entry position in PriorityQueue.

Structs

FIFO
A priority wrapper where newer entries with the same priority are placed after existing entries.
LIFO
A priority wrapper where newer entries with the same priority are placed before existing entries.
LowestFirst
Reverses the result of Ord::cmp for O. Less -> Greater and Greater -> Less.

Traits

Priority
Trait used by PriorityQueue to determine the order by which nodes should be enqueued.

Type Aliases

HighestFirst
A redundant type alias existing to have parity with LowestFirst (as the default Priority impl for Ord is highest first)