Module arena

Module arena 

Provides ArenaQueue - a PriorityQueue where entries are placed in an “arena.”

Instead of having many separate allocations like BoxQueue, this places all entries in a single, sequential block of memory to leverage cpu cache more effectively.

Structs

ArenaQueue
An arena queue - instead of having each node as a separate allocation, a block of memory is allocated for all of the nodes.
ArenaQueueHandle
An opaque handle to an ArenaQueue entry.
DualLinkArenaQueueNode
A node for an ArenaQueue which links to both the previous and next nodes.
SharedArenaQueueHandle
An opaque “shared” handle to an ArenaQueue entry.
SingleLinkArenaQueueNode
A node for an ArenaQueue which links only to the next node.

Type Aliases

DualLinkArenaQueue
A dual linked ArenaQueue.
SingleLinkArenaQueue
A single linked (next only) arena queue.