ArenaQueue

Struct ArenaQueue 

pub struct ArenaQueue<N: ArenaQueueNode> { /* private fields */ }
An arena queue - instead of having each node as a separate allocation, a block of memory is allocated for all of the nodes.

The block will never shrink, but will grow to accommodate more slots as needed.

Currently, the amount of slots allocated is always a power of two and is doubled when capacity is exceeded.

If this is an issue, BoxQueue can be used with the box-queue feature flag enabled.

It’s likely easier to use this via the type aliases SingleLinkArenaQueue and DualLinkArenaQueue.