pub struct DualLinkArenaQueueNode<P: Priority> { /* private fields */ }Expand description
A node for an ArenaQueue which links to both the previous and next nodes.
This is likely a bit verbose for normal use; the DualLinkArenaQueue type alias is recommended instead.