Provides BoxQueue - where entries are each allocated via Box.
This is usually slower than ArenaQueue, however it is usually a bit more memory efficient, as nodes are deallocated when they are removed.
Structs
- BoxQueue
- A queue where nodes are independently allocated via Box.
- BoxQueue
Handle - An opaque handle to a BoxQueue node.
- Dual
Link BoxQueue Node - A node for BoxQueue which links to both the previous and next nodes.
- Shared
BoxQueue Handle - An opaque “shared” handle to a BoxQueue node.
- Single
Link BoxQueue Node - A node for BoxQueue which only links to next.
Type Aliases
- Dual
Link BoxQueue - A dual linked BoxQueue.
- Single
Link BoxQueue - A single linked (next only) BoxQueue.