pub struct DualLinkBoxQueueNode<P: Priority> { /* private fields */ }Expand description
A node for BoxQueue which links to both the previous and next nodes.
Direct usuage is likely to be a bit verbose; usage of the DualLinkBoxQueue is recommended.