InsufficientPermitsError

Struct InsufficientPermitsError 

pub struct InsufficientPermitsError { /* private fields */ }
Error returned when we lack sufficient permits to perform an operation.

Contains the requested count of permits via requested and optionally the total available when the request failed via total.

Note that total may be None if the requester was evicted after being queued due to insufficient capacity (at which point, we don’t know what the total permits for the queue was)

Implementations

Source

impl InsufficientPermitsError

pub fn total(&self) -> Option<usize>

The total amount of permits which were available when the waiter was rejected (or None if we don’t know)

pub fn requested(&self) -> usize

The amount of permits requested