SemaphorePermit

Struct SemaphorePermit 

pub struct SemaphorePermit<'a, P: Priority, Q: SemaphoreQueue<P>>(/* private fields */);
A permit (or collection of permits) from a Semaphore

Acquired via Semaphore::acquire and associated fns.

Implementations

Source

impl<'a, P: Priority, Q: SemaphoreQueue<P>> SemaphorePermit<'a, P, Q>

pub fn evicted(&mut self) -> impl Future<Output = ()>

Returns a future which resolves when / if a higher priority requester is waiting for permit acquisition. Available only if the evict flag is enabled.

Cancel Safety: This function is cancel safe.

pub fn forget(self)

Remove these permits from the pool of permits in the semaphore.

pub fn permits(&self) -> usize

The count of permits held

pub fn belongs_to(&self, semaphore: &Semaphore<P, Q>) -> bool

Checks if self belongs to semaphore.

pub fn split(&mut self, count: usize) -> Result<Self, InsufficientPermitsError>
where P: Clone,

Split into multiple permit guards / holder. The new guard will have the same priority for evictions.

P must implement Clone for this to succeed. If P is not Clone, split_with_priority may be used instead.

pub fn split_with_priority( &mut self, count: usize, priority: P, ) -> Result<Self, InsufficientPermitsError>

pub fn merge(&mut self, other: Self) -> Result<(), ()>

Merge another permit into this one. Returns an error if the other node belongs to another semaphore

Panics if the sum of their permits would exceed the max permit count