Provides ArenaQueue - a PriorityQueue where entries are placed in an “arena.”
Instead of having many separate allocations like BoxQueue, this places all entries in a single, sequential block of memory to leverage cpu cache more effectively.
Structs
- Arena
Queue - An arena queue - instead of having each node as a separate allocation, a block of memory is allocated for all of the nodes.
- Arena
Queue Handle - An opaque handle to an ArenaQueue entry.
- Dual
Link Arena Queue Node - A node for an ArenaQueue which links to both the previous and next nodes.
- Shared
Arena Queue Handle - An opaque “shared” handle to an ArenaQueue entry.
- Single
Link Arena Queue Node - A node for an ArenaQueue which links only to the next node.
Type Aliases
- Dual
Link Arena Queue - A dual linked ArenaQueue.
- Single
Link Arena Queue - A single linked (next only) arena queue.