BoxQueue

Struct BoxQueue 

pub struct BoxQueue<N: BoxQueueNode> { /* private fields */ }
A queue where nodes are independently allocated via Box.

This is usually much slower than arena queue, but is more memory efficient as the memory used for each node is deallocated once the node is removed.

Usage is only recommended when memory efficiency is needed. If possible, DualLinkBoxQueue should be preferred over SingleLinkBoxQueue, as it almost always faster.

(difference being that DualLinkBoxQueueNode has an extra usize worth of bytes per node)