pub struct InsufficientPermitsError { /* private fields */ }Expand description
Error returned when we lack sufficient permits to perform an operation.
-
With the
semaphore-totalflag: Semaphore lacks sufficient permits when using acquire_within_total -
When more permits were requested via SemaphorePermit::split.than the permit contains.
Contains the requested count of permits via requested and optionally the total available when the request failed via total.
Note that total may be None if the requester was evicted after being queued due to insufficient capacity (at which point, we don’t know what the total permits for the queue was)
Implementations