pub struct SemaphorePermit<'a, P: Priority, Q: SemaphoreQueue<P>>(/* private fields */);Expand description
A permit (or collection of permits) from a Semaphore
Acquired via Semaphore::acquire and associated fns.
Implementations
Sourceimpl<'a, P: Priority, Q: SemaphoreQueue<P>> SemaphorePermit<'a, P, Q>
impl<'a, P: Priority, Q: SemaphoreQueue<P>> SemaphorePermit<'a, P, Q>
Sourcepub fn evicted(&mut self) -> impl Future<Output = ()>
pub fn evicted(&mut self) -> impl Future<Output = ()>
Returns a future which resolves when / if a higher priority requester is waiting for permit
acquisition. Available only if the evict flag is enabled.
Cancel Safety: This function is cancel safe.
Sourcepub fn belongs_to(&self, semaphore: &Semaphore<P, Q>) -> bool
pub fn belongs_to(&self, semaphore: &Semaphore<P, Q>) -> bool
Checks if self belongs to semaphore.
Sourcepub fn split(&mut self, count: usize) -> Result<Self, InsufficientPermitsError>where
P: Clone,
pub fn split(&mut self, count: usize) -> Result<Self, InsufficientPermitsError>where
P: Clone,
Split into multiple permit guards / holder. The new guard will have the same priority for evictions.
P must implement Clone for this to succeed. If P is not Clone,
split_with_priority may be used instead.
pub fn split_with_priority( &mut self, count: usize, priority: P, ) -> Result<Self, InsufficientPermitsError>